// *******************************************************************
// File    : ThumbNail.cs 
// Author  : Igor Tolmachev
// Email   : info@itsamples.com
// Web     : www.itsamples.com 
// You are free to use/modify this code but leave this header intact.
// *******************************************************************

using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Imaging;
using System.Data;
using System.Windows.Forms;
using System.IO;
using System.Reflection;
using Microsoft.Win32;

namespace ExifDataChanger
{
	/// <summary>
	/// Summary description for ThumbNail.
	/// </summary>
	
	public class ThumbNail : System.Windows.Forms.UserControl
	{
		private System.Windows.Forms.PictureBox pictureBox;
		private System.Windows.Forms.Label labelFileName;
		private System.Windows.Forms.Label labelImageDate;
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		#region Fields
		private string m_strPath = ""; //     
		private System.Drawing.Image m_Image = null;
		#endregion

		#region Properties
		public string FilePath
		{
			get { return m_strPath; }
		}
		public System.Drawing.Image Image
		{
			get { return m_Image; }
		}
		#endregion

		public ThumbNail(string strPath)
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			//      
			m_strPath = strPath;
			if(!strPath.Equals("")) 
			{
				//  
				m_Image = System.Drawing.Image.FromFile(strPath);
			
				int nWidth = m_Image.Width;
				int nHeight = m_Image.Height;
				int nMax = Math.Min(this.pictureBox.Width, this.pictureBox.Height);
				
				//           
				if(m_Image.Width > nMax || m_Image.Height > nMax)
				{
					if(m_Image.Width > m_Image.Height)
					{
						nWidth = nMax;
						nHeight = (int) (m_Image.Height * nMax / m_Image.Width);
					}
					else
					{
						nWidth = (int) (m_Image.Width * nMax / m_Image.Height);
						nHeight = nMax;
					}
				}
				
				//     :
				//    DTOriginal,    ,
				//    ,    
				this.pictureBox.Image = new Bitmap(m_Image, nWidth, nHeight);
				this.labelFileName.Text = Path.GetFileName(strPath);
				this.labelImageDate.Text = string.Format("{0} x {1}", m_Image.Size.Width, m_Image.Size.Height);
				this.GetImageDate(strPath);
			}
		}

		//         ReplaceBytie(...),    ,   .
		public bool UpdateDate(string strDate, bool bUpdateDateTime, bool bUpdateDateTimeOriginal, bool bUpdateDateTimeDigitized, bool bCreateBackups)
		{
			if(strDate.Length!=19) return false; // 0000:00:00 00:00:00

			int[] propIds = m_Image.PropertyIdList;
							
			bool bDateTime          = false;
			bool bDateTimeOriginal  = false;
			bool bDateTimeDigitized = false;
			
			//     	
			foreach(int id in propIds) 
			{
				switch(id)
				{
					case 306:   // DateTime
						bDateTime = true;
					break;
					case 36867: // DateTimeOriginal
						bDateTimeOriginal = true;
						break;
					case 36868: // DateTimeDigitized
						bDateTimeDigitized = true;
						break;
				}
			}
			
			//            
			if(bDateTime || bDateTimeOriginal || bDateTimeDigitized)
			{
				int nStep = 1;
				string strDateTime = "";
				string strDateTimeOriginal = "";
				string strDateTimeDigitized = "";

				//     
				if(bCreateBackups) File.Copy(m_strPath, m_strPath+".bak",true);
				
				//    /  ,     
				if(bDateTime)
				{
					PropertyItem prop = m_Image.GetPropertyItem(306);
					System.Text.ASCIIEncoding encoding = new System.Text.ASCIIEncoding();
					strDateTime = encoding.GetString(prop.Value, 0, prop.Len - 1);
					if(!strDateTime.Equals("") && bUpdateDateTime) ReplaceBytie(strDateTime, strDate, 1);
				}
				
				//    /  ,     
				if(bDateTimeOriginal)
				{
					PropertyItem prop = m_Image.GetPropertyItem(36867);
					System.Text.ASCIIEncoding encoding = new System.Text.ASCIIEncoding();
					strDateTimeOriginal = encoding.GetString(prop.Value, 0, prop.Len - 1);
					
					if(!strDateTimeOriginal.Equals("") && bUpdateDateTimeOriginal)
					{
						if(bDateTime)
							if((strDateTimeOriginal.Equals(strDate) && bUpdateDateTime) || (strDateTimeOriginal.Equals(strDateTime) && !bUpdateDateTime)) nStep += 1;

						ReplaceBytie(strDateTimeOriginal, strDate, nStep);
					}
				}
				
				//    /  ,     
				if(bDateTimeDigitized)
				{
					PropertyItem prop = m_Image.GetPropertyItem(36868);
					System.Text.ASCIIEncoding encoding = new System.Text.ASCIIEncoding();
					strDateTimeDigitized = encoding.GetString(prop.Value, 0, prop.Len - 1);
					
					if(!strDateTimeDigitized.Equals("") && bUpdateDateTimeDigitized)
					{
						nStep = 1;
						if(bDateTime)
							if((strDateTimeDigitized.Equals(strDate) && bUpdateDateTime) || (strDateTimeDigitized.Equals(strDateTime) && !bUpdateDateTime)) nStep += 1;
						if(bDateTimeOriginal)
							if((strDateTimeDigitized.Equals(strDate) && bUpdateDateTimeOriginal) || (strDateTimeDigitized.Equals(strDateTimeOriginal) && !bUpdateDateTimeOriginal)) nStep += 1;
						
						ReplaceBytie(strDateTimeDigitized, strDate, nStep);
					}
				}
	
				//    ,     
				this.pictureBox.BackColor = Color.FromArgb(128, 250, 128);
				this.labelImageDate.Text = strDate;
			}

			return true;
		}

		//    
		public bool UpdateFileDate(string strDate)
		{
			if(File.Exists(m_strPath))
			{
				if(m_Image != null) m_Image.Dispose();

				string strFormat = "yyyy:MM:dd HH:mm:ss";
				DateTime dtUpdate = DateTime.ParseExact(strDate, strFormat, null);
				
				File.SetLastWriteTime(m_strPath, dtUpdate);
				File.SetCreationTime(m_strPath, dtUpdate);
				
				m_Image = System.Drawing.Image.FromFile(m_strPath);
				return true;
			}

			return false;
		}

		//     EXIF  
		//      
		//  ,         
		public bool InsertDate(string strDate, int nTagId, bool bCreateBackups, int nLanguage)
		{
			if(strDate.Length!=19) return false; // 0000:00:00 00:00:00

			int[] propIds = m_Image.PropertyIdList;
							
			bool bDateTime  = false;
			
			//      :
			//  ,      ,   -    
			foreach(int id in propIds) 
			{
				if(id == nTagId)
				{
					bDateTime = true;
					break;
				}
			}
			
			if(bDateTime)
			{
				if(nLanguage == 0) MessageBox.Show("Target image property already exists","EXIF date Changer", MessageBoxButtons.OK, MessageBoxIcon.Warning);
				else MessageBox.Show("     ","EXIF date Changer", MessageBoxButtons.OK, MessageBoxIcon.Warning);
				return false;
			}

			//      ,   
			System.Reflection.ConstructorInfo ci = typeof(PropertyItem).GetConstructor(BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.Public , null, new Type[] { }, null);
			
			PropertyItem propItem = (PropertyItem)ci.Invoke(null);
			propItem.Id = nTagId;
			propItem.Len = 20;
			propItem.Type = 2;
			char [] szDate = strDate.ToCharArray();
			byte [] byDate = new byte[20];

			for(int i=0;i<strDate.Length;i++) byDate[i] = (byte)szDate[i];
			propItem.Value = byDate;
			
			string strTempPath = Path.GetTempFileName();
			
			//     
			if(bCreateBackups) File.Copy(m_strPath, m_strPath+".bak",true);

			m_Image.SetPropertyItem(propItem);
			m_Image.Save(strTempPath);

			m_Image.Dispose();

			File.Copy(strTempPath, m_strPath, true);
			File.Delete(strTempPath);

			m_Image = System.Drawing.Image.FromFile(m_strPath);
			if(nTagId == 36867) this.pictureBox.BackColor = Color.FromArgb(128, 250, 128);
			this.labelImageDate.Text = strDate;
			
			return true;
		}

		//     EXIF  
		//      
		//  ,         
		public bool DeleteDate(int nTagId, bool bCreateBackups, int nLanguage)
		{
			int[] propIds = m_Image.PropertyIdList;
							
			bool bDateTime  = false;
							
			//     :
			//  ,      ,   -  
			foreach(int id in propIds) 
			{
				if(id == nTagId)
				{
					bDateTime = true;
					break;
				}
			}
			
			if(!bDateTime)
			{
				if(nLanguage == 0) MessageBox.Show("Target picture does not include deleting property","EXIF date Changer", MessageBoxButtons.OK, MessageBoxIcon.Warning);
				else MessageBox.Show("    ","EXIF date Changer", MessageBoxButtons.OK, MessageBoxIcon.Warning);
				return false;
			}

			string strTempPath = Path.GetTempFileName();

			//     
			if(bCreateBackups) File.Copy(m_strPath, m_strPath+".bak",true);

			m_Image.RemovePropertyItem(nTagId);
			m_Image.Save(strTempPath);

			m_Image.Dispose();

			File.Copy(strTempPath, m_strPath, true);
			File.Delete(strTempPath);

			m_Image = System.Drawing.Image.FromFile(m_strPath);
			if(nTagId == 36867) this.pictureBox.BackColor = Color.FromArgb(255,255,255);

			return true;
		}
		
		//  
		//        /
		private void ReplaceBytie(string strWhat, string strWith, int nStep)
		{
			int nWhatLen = strWhat.Length;
			int nWithLen = strWith.Length;
			if(nWhatLen<1 || nWithLen<1) return;

			int k=0;
			byte[] byWhat = new Byte[20];
			byte[] byWith = new Byte[20];
			for(k=0; k<strWhat.Length; k++) byWhat[k]=(byte)strWhat[k];
			for(k=0; k<strWith.Length; k++) byWith[k]=(byte)strWith[k];

			if(File.Exists(m_strPath))
			{
				if(m_Image != null) m_Image.Dispose();

				FileStream stream = new FileStream(m_strPath, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);

				long nLen = stream.Length;
				long i = 0;
				long j = 0;
				byte[] byBuffer = new Byte[nLen+1];
				
				stream.Lock(0,stream.Length);
				stream.Read(byBuffer,0,(int)nLen);
				
				bool bFind = true;
				int nLocalStep = 0;
				for(i=0;i<nLen;i++)
				{
					bFind = true;
					for(j=0; j<nWhatLen; j++)
					{
						if(byBuffer[i+j]!= byWhat[j])
						{
							bFind = false;
							break;
						}
					}
					if(bFind)
					{
						nLocalStep++;
						if(nLocalStep == nStep)
						{
							stream.Seek(i,SeekOrigin.Begin);
							for(j=0; j<nWithLen; j++) stream.WriteByte(byWith[j]);
							break;
						}
						else continue;
					}
				}

				stream.Unlock(0,stream.Length);
				stream.Close();

				m_Image = System.Drawing.Image.FromFile(m_strPath);
			}
		}

		//         / DTOriginal,
		//      
		private void GetImageDate(string strFile)
		{
			int[] propIds = m_Image.PropertyIdList;

			foreach(int id in propIds) 
			{
				try 
				{
					if(id==36867) // DTOriginal
					{
						string strDate;
						
						PropertyItem prop = m_Image.GetPropertyItem(id);
						System.Text.ASCIIEncoding encoding = new System.Text.ASCIIEncoding();
						strDate = encoding.GetString(prop.Value, 0, prop.Len - 1);

						this.labelImageDate.Text = strDate;
						if(strDate.Substring(0,4).Equals("0000")) this.pictureBox.BackColor = Color.FromArgb(250, 128, 128);
						else this.pictureBox.BackColor = Color.FromArgb(128, 250, 128);

						break;
					}
				}
				catch(Exception) 
				{
				}
			}
		}

		//           ,
		// c      .
		private void OnThumbClick(object sender, System.EventArgs e)
		{
			System.IntPtr hWnd = this.ParentForm.Handle;
			
			Win32.COPYDATASTRUCT cd;
			cd.lpData = m_strPath;
			cd.dwData = 0;
			cd.cbData= 0;

			Win32.SendMessage(hWnd,0x004A,0, ref cd); //WM_COPYDATA
		}
		
		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(m_Image != null)
					m_Image.Dispose();
			}

			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.pictureBox = new System.Windows.Forms.PictureBox();
			this.labelFileName = new System.Windows.Forms.Label();
			this.labelImageDate = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// pictureBox
			// 
			this.pictureBox.BackColor = System.Drawing.Color.White;
			this.pictureBox.Cursor = System.Windows.Forms.Cursors.Hand;
			this.pictureBox.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.pictureBox.Location = new System.Drawing.Point(1, 1);
			this.pictureBox.Name = "pictureBox";
			this.pictureBox.Size = new System.Drawing.Size(128, 128);
			this.pictureBox.SizeMode = System.Windows.Forms.PictureBoxSizeMode.CenterImage;
			this.pictureBox.TabIndex = 2;
			this.pictureBox.TabStop = false;
			this.pictureBox.Click += new System.EventHandler(this.OnThumbClick);
			// 
			// labelFileName
			// 
			this.labelFileName.BackColor = System.Drawing.Color.White;
			this.labelFileName.ForeColor = System.Drawing.Color.Black;
			this.labelFileName.Location = new System.Drawing.Point(1, 129);
			this.labelFileName.Name = "labelFileName";
			this.labelFileName.Size = new System.Drawing.Size(128, 20);
			this.labelFileName.TabIndex = 3;
			this.labelFileName.Text = "Filename";
			this.labelFileName.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// labelImageDate
			// 
			this.labelImageDate.BackColor = System.Drawing.Color.White;
			this.labelImageDate.Font = new System.Drawing.Font("Arial", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(204)));
			this.labelImageDate.Location = new System.Drawing.Point(1, 149);
			this.labelImageDate.Name = "labelImageDate";
			this.labelImageDate.Size = new System.Drawing.Size(128, 20);
			this.labelImageDate.TabIndex = 4;
			this.labelImageDate.Text = "Image Date";
			this.labelImageDate.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// ThumbNail
			// 
			this.BackColor = System.Drawing.Color.Navy;
			this.Controls.Add(this.labelImageDate);
			this.Controls.Add(this.labelFileName);
			this.Controls.Add(this.pictureBox);
			this.Name = "ThumbNail";
			this.Size = new System.Drawing.Size(130, 170);
			this.ResumeLayout(false);

		}
		#endregion
	}
}
